@@ -168,12 +168,11 @@ class TableWriter(object):
                 data = pd.read_parquet(path_input, **read_from_file_args)
             else:
                 raise ValueError(f"Can not read file {path_input} : unsupported extension.")
-            print(path_input)
 
         if "dask.dataframe" in str(type(data)):
             data = data.head(len(data.index))
 
-        if not isinstance(data, pd.DataFrame):
+        if data is not None and not isinstance(data, pd.DataFrame):
             raise ValueError(f"Data must be a DataFrame, got {type(data)}")
 
         if packages is None:
@@ -204,7 +203,12 @@ class TableWriter(object):
         if self.label is not None:
             self.to_latex_args["label"] = self.label
         if "column_format" not in self.to_latex_args:
-            self.to_latex_args["column_format"] = "|l|" + len(self.data.columns) * "c" + "|"
+            if isinstance(self.data.index, pd.MultiIndex):
+                self.to_latex_args["column_format"] = (
+                    "|" + self.data.index.nlevels * "l" + "|" + len(self.data.columns) * "c" + "|"
+                )
+            else:
+                self.to_latex_args["column_format"] = "|l|" + len(self.data.columns) * "c" + "|"
         if "escape" not in self.to_latex_args:
             self.to_latex_args["escape"] = True
         if "longtable" not in self.to_latex_args:
@@ -243,9 +247,31 @@ class TableWriter(object):
 
     # noinspection PyUnresolvedReferences
     @property
-    def path(self) -> Union[str, Path, "TransparentPath"]:
+    def path(self) -> Union[Path, "TransparentPath"]:
         return self.__path
 
+    # noinspection PyUnresolvedReferences
+    @property
+    def path_output(self) -> Union[Path, "TransparentPath"]:
+        return self.__path
+
+    # noinspection PyUnresolvedReferences
+    @path.setter
+    def path(self, apath: Union[str, Path, "TransparentPath", None]):
+        if type(apath) == str:
+            if self.__path is not None:
+                apath = type(self.__path)(apath)
+            else:
+                apath = Path(apath)
+        if apath.suffix != ".tex":
+            apath = apath.with_suffix(".tex")
+        self.__path = apath
+
+    # noinspection PyUnresolvedReferences
+    @path_output.setter
+    def path_output(self, apath: Union[str, Path, "TransparentPath", None]):
+        self.path = apath
+
     # ////////////
     # // Makers //
     # ////////////
@@ -255,20 +281,17 @@ class TableWriter(object):
         self.paperwidth = paperwidth
         self.paperheight = paperheight
 
-        if self.paperwidth != 0 and self.paperheight != 0:
-            return
-
         # Try to guess a kind of optimal width for the table
-        if not self.data.empty:
+        if not self.data.empty and self.paperwidth == 0:
+
             charswidth = (
-                len("".join(list(self.data.columns.dropna().astype(str))))
-                + max([len(ind) for ind in self.data.index.dropna().astype(str)])
+                len("".join(list(self.data.columns.dropna().astype(str)))) + max([len(str(e)) for e in self.data.index])
             ) * 0.178
             self.paperwidth = charswidth + 0.8 * (len(self.data.columns)) + 1
             if self.paperwidth < 9:
                 self.paperwidth = 9
         # Same for height
-        if not self.data.empty:
+        if not self.data.empty and self.paperheight == 0:
             self.paperheight = 3.5 + (len(self.data.index)) * 0.45
             if self.paperheight < 4:
                 self.paperheight = 4
@@ -322,8 +345,8 @@ class TableWriter(object):
         if self.caption is not None and self.hide_numbering:
             self.body = self.body.replace("\\caption{", "\\caption*{")
 
-        if self.caption is not None or self.label is not None:
-            self.body = self.body.replace("\n\\toprule", "\\\\\n\\toprule")
+        # if self.caption is not None or self.label is not None:
+        #     self.body = self.body.replace("\n\\toprule", "\\\\\n\\toprule")
         self.body = self.body.replace("\\\\\\\\", "\\\\")
 
     def _make_footer(self) -> None:
@@ -371,8 +394,21 @@ class TableWriter(object):
     def build(self):
         """build header body and footer."""
         if "escape" in self.to_latex_args and self.to_latex_args["escape"]:
-            self.data.index = [self._escape_special_chars(s) for s in self.data.index]
-            self.data.columns = [self._escape_special_chars(s) for s in self.data.columns]
+            if isinstance(self.data.index, pd.MultiIndex):
+                names = [self._escape_special_chars(s) for s in self.data.index.names]
+                # noinspection PyTypeChecker
+                self.data.index = pd.MultiIndex.from_tuples(
+                    [[self._escape_special_chars(s) for s in l_] for l_ in self.data.index]
+                )
+                self.data.index.names = names
+            else:
+                name = self._escape_special_chars(self.data.index.name)
+                self.data.index = [self._escape_special_chars(s) for s in self.data.index]
+                self.data.index.name = name
+            if isinstance(self.data.columns, pd.MultiIndex):
+                self.data.columns = [[self._escape_special_chars(s) for s in l_] for l_ in self.data.columns]
+            else:
+                self.data.columns = [self._escape_special_chars(s) for s in self.data.columns]
             self.data = self.data.applymap(self._escape_special_chars)
         self.to_latex_args["escape"] = False
         self._make_header()
@@ -392,7 +428,11 @@ class TableWriter(object):
 
     # noinspection StandardShellInjection
     def compile(
-        self, silenced: bool = True, recreate: bool = True, clean: bool = True, clean_tex: bool = False,
+        self,
+        silenced: bool = True,
+        recreate: bool = True,
+        clean: bool = True,
+        clean_tex: bool = False,
     ) -> None:
         """Compile the pdf.
 
@@ -421,18 +461,20 @@ class TableWriter(object):
             raise ValueError(f"Tex file {self.__path} not found.")
 
         path_to_compile = self.__path
+        # noinspection PyUnresolvedReferences
         if "fs_kind" in dir(self.__path) and "gcs" in self.__path.fs_kind:  # Using TransparentPath on gcs
             path_to_compile = tempfile.NamedTemporaryFile(delete=False, suffix=".tex")
             path_to_compile.close()
+            # noinspection PyUnresolvedReferences
             self.__path.get(path_to_compile.name)
-            path_to_compile = Path(path_to_compile.name, fs="local")
+            path_to_compile = type(self.__path)(path_to_compile.name, fs="local")
 
         command = "pdflatex -synctex=1 -interaction=nonstopmode "
         parent = path_to_compile.parent
         if parent != ".":
-            command = f"{command} -output-directory=\"{parent}\""
+            command = f'{command} -output-directory="{parent}"'
 
-        command = f"{command} \"{path_to_compile}\""
+        command = f'{command} "{path_to_compile}"'
         if silenced:  # unix
             if os.name == "posix":
                 command = f"{command} > /dev/null"
@@ -444,11 +486,14 @@ class TableWriter(object):
         time.sleep(0.5)
         x3 = os.system(command)
 
+        # noinspection PyUnresolvedReferences
         if "fs_kind" in dir(self.__path) and "gcs" in self.__path.fs_kind:
             glob_in = path_to_compile.parent
             for path in glob_in.glob(f"{path_to_compile.stem}*"):
                 path_gcs = self.__path.with_suffix(path.suffix)
+                # noinspection PyUnresolvedReferences
                 path.put(path_gcs)
+                # noinspection PyUnresolvedReferences
                 path.rm()
 
         if x1 != 0 or x2 != 0 or x3 != 0:
@@ -479,6 +524,7 @@ class TableWriter(object):
                 if f.is_file():
                     f.unlink()
                 elif "rm" in dir(f):
+                    # noinspection PyUnresolvedReferences
                     f.rm()
                 else:
                     shutil.rmtree(f)
@@ -529,7 +575,10 @@ def set_color(obj: Any, color: str) -> str:
 
 # noinspection PyTypeChecker
 def set_color_dataframe(
-    df: Union[pd.DataFrame, pd.Series], color: str, color_index: bool = False, color_columns: bool = False,
+    df: Union[pd.DataFrame, pd.Series],
+    color: str,
+    color_index: bool = False,
+    color_columns: bool = False,
 ) -> Union[pd.DataFrame, pd.Series]:
     r"""Sets color for the entier DataFrame's or Series's entries.
 
